/*
 * rmote.c
 *
 * Copyright (c) 2012, ROSAND Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the Contiki operating system.
 *
 */

#include <stdio.h>
#include <string.h>

#include "contiki.h"
#include "net/packetbuf.h"
#include "net/rime/rimestats.h"
#include "net/netstack.h"
#include "dev/leds.h"

#include "rmote.h"
#include "config.h"
#include "dev/spi-arch.h"
#include "dev/i2c.h"

static uint8_t on_off;
static uint8_t channel;
static uint8_t power;

/*---------------------------------------------------------------------------*/
PROCESS(rmote_radio_process, "R-Mote SPI Vitrual Radio Driver");
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
static int
init(void)
{
  spi_init();
  process_start(&rmote_radio_process, NULL);
  return 0;
}
/*---------------------------------------------------------------------------*/
static int
prepare(const void *payload, unsigned short payload_len)
{
  spi_write((uint8_t *) payload, payload_len);
  return payload_len;
}
/*---------------------------------------------------------------------------*/
static int
transmit(unsigned short transmit_len)
{
  uint16_t tmp;

  tmp = transmit_len;
  config_set(CONFIG_RF_XMIT, (uint8_t *) &tmp, 2);
  config_request(CONFIG_RF_XMIT, (uint8_t *) &tmp, 2);
  return tmp;
}
/*---------------------------------------------------------------------------*/
static int
send(const void *payload, unsigned short payload_len)
{
  prepare(payload, payload_len);
  return transmit(payload_len);
}
/*---------------------------------------------------------------------------*/
static int
read(void *buf, unsigned short bufsize)
{
  return spi_read(buf, bufsize);
}
/*---------------------------------------------------------------------------*/
static int
cca(void)
{
  return 0;
}
/*---------------------------------------------------------------------------*/
static int
receiving_packet(void)
{
  return 0;
}
/*---------------------------------------------------------------------------*/
static int
pending_packet(void)
{
  return 0;
}
/*---------------------------------------------------------------------------*/
static int
on(void)
{
  on_off = 1;
  config_set(CONFIG_RF_ON, (uint8_t *) &on_off, 1);
  return 0;
}
/*---------------------------------------------------------------------------*/
uint8_t
rmote_get_channel(void)
{
  return channel;
}
/*---------------------------------------------------------------------------*/
void
rmote_set_channel(uint8_t c)
{
  channel = c;
  config_set(CONFIG_RF_CHAN, (uint8_t *) &channel, 1);
}
/*---------------------------------------------------------------------------*/
void
rmote_listen_channel(uint8_t c)
{

}
/*---------------------------------------------------------------------------*/
void
rmote_set_pan_addr(uint16_t pan, uint8_t *ieee_addr)
{
  config_request(CONFIG_MACADDR, ieee_addr, 8);
  config_set(CONFIG_PANID, (uint8_t *) &pan, 2);
}
/*---------------------------------------------------------------------------*/
void
rmote_set_txpower(uint8_t p)
{
  power = p;
  config_set(CONFIG_RF_POWER, (uint8_t *) &p, 1);
}
/*---------------------------------------------------------------------------*/
uint8_t
rmote_get_txpower(void)
{
  return power;
}
/*---------------------------------------------------------------------------*/
uint8_t
rmote_get_raw_rssi(void)
{
  uint8_t rssi;

  config_request(CONFIG_MACADDR, &rssi, 1);
  return rssi;
}
/*---------------------------------------------------------------------------*/
static int
off(void)
{
  on_off = 0;
  config_set(CONFIG_RF_OFF, &on_off, 1);
  return 0;
}
/*---------------------------------------------------------------------------*/

PROCESS_THREAD(rmote_radio_process, ev, data)
{
  int len;

  PROCESS_BEGIN();

  while(1) {
    PROCESS_WAIT_EVENT();
    packetbuf_clear();
    len = read(packetbuf_dataptr(), PACKETBUF_SIZE);
    if(len > 0) {
      packetbuf_set_datalen(len);
      NETSTACK_RDC.input();
    }
  }

  PROCESS_END();
}

const struct radio_driver rmote_driver =
  {
    init,
    prepare,
    transmit,
    send,
    read,
    cca,
    receiving_packet,
    pending_packet,
    on,
    off
  };
