/*
 * env-process.c
 *
 * Copyright (c) 2012, ROSAND Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the Contiki operating system.
 *
 */

#include "contiki.h"

#include "dev/rmote-sensors.h"
#include "env-process.h"
#include "defs.h"
#include "leds.h"

static uint16_t temp, humid, lux;

/*---------------------------------------------------------------------------*/
PROCESS(env_sensor_process, "environment server process");
/*---------------------------------------------------------------------------*/
static void
init(void)
{
  temp  = 0;
  humid = 0;
  lux   = 0;

  SENSORS_ACTIVATE(env_sensor);
  process_start(&env_sensor_process, NULL);
}
/*---------------------------------------------------------------------------*/
static uint16_t
handle(uint8_t *data)
{
  uint8_t *tmp = data;

  *tmp++ = 0;
  *tmp++ = ENV_TEMP_SENSOR;
  *tmp++ = ENV_TEMP_LENGTH;
  *tmp++ = HI_UINT16(temp);
  *tmp++ = LO_UINT16(temp);
  *tmp++ = ENV_HUMID_SENSOR;
  *tmp++ = ENV_HUMID_LENGTH;
  *tmp++ = HI_UINT16(humid);
  *tmp++ = LO_UINT16(humid);
  *tmp++ = ENV_LUX_SENSOR;
  *tmp++ = ENV_LUX_LENGTH;
  *tmp++ = HI_UINT16(lux);
  *tmp++ = LO_UINT16(lux);
  *data = tmp - data;

  return *data;
}
/*---------------------------------------------------------------------------*/
PROCESS_THREAD(env_sensor_process, ev, data)
{
  static struct etimer et;

  PROCESS_BEGIN();

  etimer_set(&et, ENV_SENSOR_POLL_INTERVAL * CLOCK_SECOND);

  while(1) {
    int vtemp, vhumid, vlux;

    PROCESS_WAIT_EVENT_UNTIL(etimer_expired(&et));
    vtemp = env_sensor.value(ENV_TEMP_SENSOR);
    vhumid = env_sensor.value(ENV_HUMID_SENSOR);
    vlux = env_sensor.value(ENV_LUX_SENSOR);
    if ((vtemp != -1) && (vhumid != -1) && (vlux != -1)) {
      int t, vt;

      t  =  temp / ENV_TEMP_SCALE;
      vt = vtemp / ENV_TEMP_SCALE;
      if ((vt != t) || (vhumid != humid) || (vlux != lux)) {
        temp = vtemp;
        humid = vhumid;
        lux = vlux;
        sensors_changed(&env_sensor);
      }
    }
    etimer_restart(&et);
  }
  PROCESS_END();
}
/*---------------------------------------------------------------------------*/

const struct sensors_handle env_sensor_handle =
{
    init,
    handle,
};
