/*
 * Copyright (c) 2005, Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the Configurable Sensor Network Application
 * Architecture for sensor nodes running the Contiki operating system.
 *
 * This is a dummy non-functional dummy implementation.
 *
 * $Id: leds-arch.c,v 1.1 2006/12/22 17:05:31 barner Exp $
 *
 * -----------------------------------------------------------------
 *
 * Author  : Adam Dunkels, Joakim Eriksson, Niclas Finne, Simon Barner
 * Created : 2005-11-03
 * Updated : $Date: 2006/12/22 17:05:31 $
 *           $Revision: 1.1 $
 */

#include <avr/io.h>
#include "dev/leds.h"

#define LEDS_DDR_GREEN                      DDRG
#define LEDS_PORT_GREEN                     PORTG
#define LEDS_DDR_RED                        DDRE
#define LEDS_PORT_RED                       PORTE
#define LEDS_RED_PIN                        PIN0       // Red
#define LEDS_GREEN_PIN                      PIN5       // Green

/*---------------------------------------------------------------------------*/
void
leds_arch_init(void)
{
  LEDS_DDR_GREEN  |=   (1 << LEDS_GREEN_PIN);
  LEDS_PORT_GREEN |= (1 << LEDS_GREEN_PIN);
  LEDS_DDR_RED  |=   (1 << LEDS_RED_PIN);
  LEDS_PORT_RED |= (1 << LEDS_RED_PIN);
}
/*---------------------------------------------------------------------------*/
unsigned char
leds_arch_get(void)
{
  uint8_t l = 0;

  if ((LEDS_PORT_RED & (1 << LEDS_RED_PIN)) == 0) {
    l |= LEDS_RED;
  }
  if ((LEDS_PORT_GREEN & (1 << LEDS_GREEN_PIN)) == 0) {
    l |= LEDS_GREEN;
  }
  return l;
}
/*---------------------------------------------------------------------------*/
void
leds_arch_set(unsigned char leds)
{
  if(leds & LEDS_RED) {
    LEDS_PORT_RED &= ~(1 << LEDS_RED_PIN);
  } else {
    LEDS_PORT_RED |= (1 << LEDS_RED_PIN);
  }
  if(leds & LEDS_GREEN) {
    LEDS_PORT_GREEN &= ~(1 << LEDS_GREEN_PIN);
  } else {
    LEDS_PORT_GREEN |= (1 << LEDS_GREEN_PIN);
  }
}
/*---------------------------------------------------------------------------*/
